%***************************
% Written by Farinaz Edalat
% May 9, 2005
%***************************

%**********************************************************
% Description: This is for COST269-Model A channel model which corresponds
% to typical office environment for NLOS conditions and 50ns rms delay
% spread
%
% (1) Interpolate the given model's power-delay-profile using
% linear interpolation b/t successive points and obtain uniform sampling of
% the resulting continuous PDP for a desired sampling time

% (2) 2 example block are codes for N=128 and N=8 are given
% not that only one block-of-code must be on (not comment-out) at a time

% Given data:*******************************
N = 8; %num_channels
Ts = 1/(32*N*1e6); %simulation sampling time

% sample at Ts's (these are sampling times of the IFFT output data (in simulation))
T_sample = 32*Ts ./ 1e-9;   %since the x-axis is scaled by 1e-9
% choose an m such that T_sample*m < max(in)
m = 3; % m = 3 for N=8  but m=50 for N=128
n = T_sample.*[1:m];
%******************************************

% ModelA's delay and average-relative-power 
delay = [0 10 20 30 40 50 60 70 80 90 110 140 170 200 240 290 340 390].*1e-9;
avg_relative_power_db = [0.0 -0.9 -1.7 -2.6 -3.5 -4.3 -5.2 -6.1 -6.9 -7.8 -4.7 -7.3 -9.9 -12.5 -13.7 -18.0 -22.4 -26.7];
variances = (10.^(avg_relative_power_db/10));
[rms_delay_spread_model, mean_excess_delay_model] = delay_spread(delay, variances)

% 1) DT to CT (using linear-inerpolator) and at the same time find samples
% of PDP at a given uniform sampling time
in = delay./1e-9;
out = variances;

P = [];
for (i=1:17)
    P = [P;  polyfit(in(i:i+1), out(i:i+1), 1)];
end
% Each row of P corresponds to each 2-point linear interpolation

out_n = [];
for (i = 1:length(n))
    k = 1;
    while (k < 18)
        if (n(i) <= in(k+1))
           out_n = [out_n polyval(P(k,:), n(i))];
           k = 18;
        elseif (n(i) > 390)
           out_n = [out_n polyval(P(17,:), n(i))]; 
           k = 18;
       else    
           k = k+1;
       end
   end
end

% (2)
%**************************************************************************
%% USE below code for N = 128
%% the mask is chosen such that the rms-delay-spread and mean-excess-delay
%% be close to those of the original model
% mask = [1 1 0 ones(1,9) 0 0 1 0 0 0 1 0 0 0 1 0 0 1 0 0 0 0 0 1 0 0 0 0 0 1 0 0 0 0 0 1 0 0 0 0 0 0 1];
% 
% n2 = T_sample.*[0:m].*1e-9;
% vars = [variances(1)  out_n];
% 
% n2_masked = mask .* n2;
% var_masked = mask .* vars;
% [rms_delay_spread, mean_excess_delay] = delay_spread(n2_masked, var_masked)
% B_5 = 1/(5*rms_delay_spread)   %coherence bandwidth for correlation function of 0.5
% B_50 = 1/(50*rms_delay_spread) %coherence bandwidth for correlation funciton of 0.9
% 
% %Plots of Spaced-frequency correlation function R(delta*f) vs. delta*f 
% %which is the FFT(PSD)
% 
% T_s1 = delay(2);
% N = 2^15;
% n = [1:N];
% k = [0:N-1];
% omega_dt = 2*pi*k/N;
% omega_ct = omega_dt ./ T_s1 ;
% freq = omega_ct ./ (2*pi);
% freq_cen1 = freq - 1/(2*T_s1);
% R1 = fftshift(fft(variances,N));
% R1 = R1 ./ max(R1);
% 
% T_s = n2_masked(2);
% N = 2^15;
% n = [1:N];
% k = [0:N-1];
% omega_dt = 2*pi*k/N;
% omega_ct = omega_dt ./ T_s ;
% freq = omega_ct ./ (2*pi);
% freq_cen = freq - 1/(2*T_s);
% R = fftshift(fft(var_masked,N));
% R = R./max(R);
% 
% figure(2)
% plot(freq_cen./1e6, abs(R),'m', freq_cen1./1e6, abs(R1),'b--')
% grid on;
% xlabel('\Delta f (MHz)')
% ylabel('Spaced-frequency correlation function')
% 
% figure(3)
% stem(delay./1e-9, variances)
% hold on;
% plot(delay./1e-9, variances, 'r', n2./1e-9, vars, 'bx', n2_masked./1e-9, var_masked, 'm*')
% hold off;
% grid on;
% xlabel('Time delay [ns]')
% ylabel('Average relative power (dB)')
% 
% 
% var_sampled = [var_masked(1)];
% delay_sampled = [0];
% delay_masked = [0:m].*32.*mask;
% 
% for (i=2:length(delay_masked))
%     if (delay_masked(i) > 0 )
%         delay_sampled = [delay_sampled delay_masked(i)];
%         var_sampled = [var_sampled vars(i)];
%     end
% end
% %
%%End of the block of codes for N=128*********************************************

%% This is the block of codes for N=8*****************
n2 = T_sample.*[0:m].*1e-9;  
vars = [variances(1)  out_n];

[rms_delay_spread_simulated, mean_excess_delay_simulated] = delay_spread(n2, vars)

figure(2)
stem(delay./1e-9, variances)
hold on;
plot(delay./1e-9, variances, 'r', n2./1e-9, vars, 'm*')
hold off;
grid on;
xlabel('Time delay [ns]')
ylabel('Average relative power (dB)')

var_sampled = vars;
delay_sampled = 32 .* [0:m];  %=n2./Ts
%%End of block of codes for N=8*****************

% Save the variances and delays to load into the ofdm_channel_cost259_model
% module in SystemOFDM library
save model_a_vars.dat var_sampled -ascii
save delay_Ts_a.dat delay_sampled -ascii

